unit ColorEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, PPControl;

type
  TColorEditForm = class(TForm)
    Type_Unused: TRadioButton;
    Type_Color: TRadioButton;
    Type_Separator: TRadioButton;
    EColorName: TEdit;
    ECharacter: TEdit;
    LCharacter: TLabel;
    LColor: TLabel;
    PColor: TPanel;
    BSelectColor: TButton;
    BPickColor: TButton;
    BApply: TBitBtn;
    BApplyNext: TBitBtn;
    LColorNumLabel: TLabel;
    LColorNum: TLabel;
    ColorPicker: TColorDialog;
    BClose: TBitBtn;
    procedure BSelectColorClick(Sender: TObject);
    procedure BCloseClick(Sender: TObject);
    procedure BApplyClick(Sender: TObject);
    procedure BApplyNextClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure BPickColorClick(Sender: TObject);
    procedure TypeClick(Sender: TObject);
  private
    { Private declarations }
    CurrentColor:integer;
  public
    { Public declarations }
    procedure MoveToColor(Index:integer);
    procedure LoadColor(Index:integer);
    procedure SaveColor;
  end;

var
  ColorEditForm: TColorEditForm;

implementation

uses Colors, PickColor;

{$R *.dfm}

procedure TColorEditForm.BSelectColorClick(Sender: TObject);
begin
  ColorPicker.Color:=PColor.Color;
  if ColorPicker.Execute then PColor.Color:=ColorPicker.Color;
end;

procedure TColorEditForm.LoadColor(Index:integer);
begin
  with PPColors[Index] do begin
    CurrentColor:=Index;
    LColorNum.Caption:=IntToStr(Index)+':';
    case EntryType of
      etUnused:begin
                 Type_Unused.Checked:=true;
                 ActiveControl:=Type_Unused;
               end;
      etSeparator:begin
                    Type_Separator.Checked:=true;
                    ActiveControl:=Type_Separator;
                  end;
    else
      Type_Color.Checked:=true;
      ActiveControl:=Type_Color;
    end;
    EColorName.Text:=Name;
    PColor.Color:=RGB2Color(Color);
    ECharacter.Text:=Character;
    TypeClick(nil);
  end;
end;

procedure TColorEditForm.SaveColor;
var s1:string[1];
begin
  PickColorForm.Hide;
  with PPColors[CurrentColor] do begin
    if Type_Unused.Checked then EntryType:=etUnused
    else if Type_Separator.Checked then EntryType:=etSeparator
    else if EntryType IN [etUnused,etSeparator] then begin
      EntryType:=etUnselected;
      Groups:=0;
    end;
    Name:=EColorName.Text;
    Color:=Color2RGB(PColor.Color);
    s1:=ECharacter.Text;
    if s1[0]>#0 then Character:=s1[1] else Character:=#32;
  end;
end;

procedure TColorEditForm.MoveToColor(Index:integer);
var BaseRect:TRect; BasePoint:TPoint; Max:integer;
begin
  if (Index<0) OR (Index>255) then exit;
  with ColorsForm do begin
    BaseRect:=ColorBox.ItemRect(Index);
    BasePoint:=ColorBox.ClientToScreen(Point(BaseRect.Left,BaseRect.Bottom));
    Max:=Top+Height;
    if BasePoint.Y>Max then BasePoint.Y:=Max;
  end;
  Max:=Screen.Height-Height;
  if BasePoint.Y>Max then BasePoint.Y:=Max;
  Left:=BasePoint.X; Top:=BasePoint.Y;
end;

procedure TColorEditForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TColorEditForm.BApplyClick(Sender: TObject);
begin
  SaveColor; Hide;
end;

procedure TColorEditForm.BApplyNextClick(Sender: TObject);
begin
  SaveColor;
  if CurrentColor<255 then begin
    ColorsForm.ColorBox.ItemIndex:=CurrentColor+1;
    MoveToColor(CurrentColor+1);
    LoadColor(CurrentColor+1);
  end else
    Hide;
end;

procedure TColorEditForm.FormHide(Sender: TObject);
begin
  PickColorForm.Hide;
end;

procedure TColorEditForm.BPickColorClick(Sender: TObject);
begin
  with PickColorForm do
    if SelectFile then Show;
end;

procedure TColorEditForm.TypeClick(Sender: TObject);
var IsColor:boolean;
begin
  IsColor:=Type_Color.Checked;
  EColorName.Enabled:=IsColor OR Type_Separator.Checked;
  LColor.Enabled:=IsColor;
  PColor.Enabled:=IsColor;
  BSelectColor.Enabled:=IsColor;
  BPickColor.Enabled:=IsColor;
  LCharacter.Enabled:=IsColor;
  ECharacter.Enabled:=IsColor;
end;

end.
