unit Colors;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ToolWin, ComCtrls, PPControl, Menus;

type
  TColorsForm = class(TForm)
    ColorToolBar: TToolBar;
    BLoadColorSet: TToolButton;
    BSaveColorSet: TToolButton;
    LoadSetPopup: TPopupMenu;
    SaveSetPopup: TPopupMenu;
    LoadFrom1: TMenuItem;
    LoadFrom2: TMenuItem;
    LoadFrom3: TMenuItem;
    LoadFrom4: TMenuItem;
    LoadFrom5: TMenuItem;
    LoadFrom6: TMenuItem;
    LoadFrom7: TMenuItem;
    LoadFrom8: TMenuItem;
    CancelLoad: TMenuItem;
    SaveTo1: TMenuItem;
    SaveTo2: TMenuItem;
    SaveTo3: TMenuItem;
    SaveTo4: TMenuItem;
    SaveTo5: TMenuItem;
    SaveTo6: TMenuItem;
    SaveTo7: TMenuItem;
    SaveTo8: TMenuItem;
    CancelSave: TMenuItem;
    ToolButton1: TToolButton;
    BMoveUp: TToolButton;
    BMoveDown: TToolButton;
    BCleanup: TToolButton;
    ColorBox: TListBox;
    ToolButton2: TToolButton;
    BClose: TToolButton;
    BEditColor: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure ColorBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ColorBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure BLoadColorSetClick(Sender: TObject);
    procedure LoadSet(Sender: TObject);
    procedure BSaveColorSetClick(Sender: TObject);
    procedure SaveSet(Sender: TObject);
    procedure BMoveUpClick(Sender: TObject);
    procedure BMoveDownClick(Sender: TObject);
    procedure BCleanupClick(Sender: TObject);
    procedure ColorBoxDblClick(Sender: TObject);
    procedure BCloseClick(Sender: TObject);
  private
    { Private declarations }
    LCB_Checked,LCB_Unchecked:TBitmap;
  public
    { Public declarations }
    procedure ColorBoxChanged;
  end;

var
  ColorsForm: TColorsForm;

implementation

uses Preview, ColorEdit;

{$R *.dfm}
{$R check.res}

procedure TColorsForm.FormCreate(Sender: TObject);
var I:integer;
begin
  for I:=0 to 255 do ColorBox.Items.Add('');
  LCB_Unchecked:=TBitmap.Create;
  LCB_Unchecked.LoadFromResourceName(HInstance,'LCB_UNCHECKED');
  LCB_Unchecked.Transparent:=true;
  LCB_Checked:=TBitmap.Create;
  LCB_Checked.LoadFromResourceName(HInstance,'LCB_CHECKED');
  LCB_Checked.Transparent:=true;
end;

const TextXOffset=48;
      CharacterXOffset=18;
      ColorBoxXOffset=30;
      PresetsWidth=68;
      SeparatorColor=$E0E0E0;

procedure TColorsForm.ColorBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var Bit:integer; Presets:string[9];
begin with ColorBox,Canvas,Rect,PPColors[Index] do begin
  case EntryType of
    etUnused:Brush.Color:=clWindow;
    etSeparator:Brush.Color:=SeparatorColor;
  end;
  FillRect(Rect);
  case EntryType of
    etUnused:begin end;
    etSeparator:begin
      Font.Color:=clWindowText;
      Font.Style:=[fsBold];
      TextOut(Left+TextXOffset,Top+2,Name);
    end;
  else
    if EntryType=etSelected then Draw(Left+2,Top+2,LCB_Checked)
                            else Draw(Left+2,Top+2,LCB_Unchecked);
    TextOut(Left+TextXOffset,Top+2,Name);
    Font.Name:='Courier New'; Font.Size:=10;
    TextOut(Left+CharacterXOffset,Top+1,Character);
    Presets:=#183#183#183#183#183#183#183#183;
    for Bit:=0 to 7 do
      if ((Groups AND (1 SHL Bit))<>0) then
        Presets[Bit+1]:=char(Bit+49);
    TextOut(Right-PresetsWidth,Top+1,Presets);
    Pen.Color:=clBlack; Pen.Style:=psSolid;
    Brush.Color:=RGB2Color(Color); Brush.Style:=bsSolid;
    Rectangle(Left+ColorBoxXOffset,Top+1,Left+ColorBoxXOffset+15,Top+16);
  end;
  Pen.Style:=psClear; Brush.Style:=bsClear;
end; end;

procedure TColorsForm.FormShow(Sender: TObject);
begin
  PreviewForm.ShowColors.Down:=true;
end;

procedure TColorsForm.FormHide(Sender: TObject);
begin
  PreviewForm.ShowColors.Down:=false;
  if Assigned(ColorEditForm) then ColorEditForm.Hide;
end;

procedure TColorsForm.FormDestroy(Sender: TObject);
begin
  LCB_Checked.Free;
  LCB_Unchecked.Free;
end;

procedure TColorsForm.ColorBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Index:integer;
begin
  if X>16 then exit;
  Index:=ColorBox.ItemAtPos(Point(X,Y),true);
  if (Index<2) OR (Index>255) then exit;
  case PPColors[Index].EntryType of
    etUnused:exit;
    etSeparator:exit;
    etSelected:PPColors[Index].EntryType:=etUnselected;
    etUnselected:PPColors[Index].EntryType:=etSelected;
  end;
  ColorBoxChanged;
end;

procedure TColorsForm.FormResize(Sender: TObject);
begin
  ColorBox.Invalidate;
end;

procedure TColorsForm.BLoadColorSetClick(Sender: TObject);
var POPos:TPoint;
begin
  POPos:=BLoadColorSet.ClientToScreen(Point(0,BLoadColorSet.Height));
  LoadSetPopup.Popup(POPos.X,POPos.Y);
end;

procedure TColorsForm.LoadSet(Sender: TObject);
var SetBit,C:integer;
begin
  SetBit:=1 SHL (Sender as TMenuItem).Tag;
  for C:=0 to 255 do with PPColors[C] do
    if EntryType IN [etSelected,etUnselected] then begin
      if (Groups AND SetBit)=0 then EntryType:=etUnselected else EntryType:=etSelected;
    end;
  ColorBoxChanged;
end;

procedure TColorsForm.ColorBoxChanged;
begin
  ColorBox.Invalidate;
  Application.ProcessMessages;
  PPControl.Changed(ctBase);
end;

procedure TColorsForm.BSaveColorSetClick(Sender: TObject);
var POPos:TPoint;
begin
  POPos:=BSaveColorSet.ClientToScreen(Point(0,BSaveColorSet.Height));
  SaveSetPopup.Popup(POPos.X,POPos.Y);
end;

procedure TColorsForm.SaveSet(Sender: TObject);
var MaskBit,SetBit,C:integer;
begin
  SetBit:=1 SHL (Sender as TMenuItem).Tag;
  MaskBit:=SetBit XOR $FF;
  for C:=0 to 255 do with PPColors[C] do
    case EntryType of
      etUnselected:Groups:=Groups AND MaskBit;
      etSelected:Groups:=(Groups AND MaskBit) OR SetBit;
    end;
  ColorBox.Invalidate;
end;

procedure TColorsForm.BMoveUpClick(Sender: TObject);
var Index:integer; Temp:TPPColor;
begin
  Index:=ColorBox.ItemIndex;
  if Index<=2 then exit;
  Temp:=PPColors[Index-1];
  PPColors[Index-1]:=PPColors[Index];
  PPColors[Index]:=Temp;
  ColorBox.ItemIndex:=Index-1;
  ColorBoxChanged;
end;

procedure TColorsForm.BMoveDownClick(Sender: TObject);
var Index:integer; Temp:TPPColor;
begin
  Index:=ColorBox.ItemIndex;
  if (Index<2) OR (Index>=255) then exit;
  Temp:=PPColors[Index+1];
  PPColors[Index+1]:=PPColors[Index];
  PPColors[Index]:=Temp;
  ColorBox.ItemIndex:=Index+1;
  ColorBoxChanged;
end;

procedure TColorsForm.BCleanupClick(Sender: TObject);
var A,B:integer;
begin
  B:=1;
  for A:=2 to 255 do
    if PPColors[A].EntryType<>etUnused then begin
      inc(B);
      if A<>B then PPColors[B]:=PPColors[A];
    end;
  while B<255 do begin
    inc(B);
    PPColors[B].EntryType:=etUnused;
  end;
  ColorBox.ItemIndex:=-1;
  ColorBoxChanged;
end;

procedure TColorsForm.ColorBoxDblClick(Sender: TObject);
var Index:integer;
begin
  Index:=ColorBox.ItemIndex;
  if Index<2 then begin
    ColorEditForm.Hide;
    exit;
  end;
  with ColorEditForm do begin
    MoveToColor(Index);
    LoadColor(Index);
    Show;
  end;
end;

procedure TColorsForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

end.
