unit PPCore;
interface
uses SysUtils, Graphics, ScaleBMP, PPControl;

procedure PPCore_Resize;
procedure PPCore_Transform;
procedure PPCore_Reduce;
procedure PPCore_Finish;

procedure SetColorRamp(Factor,Brightness,Contrast,Gamma,Saturation:integer);

implementation
uses ImageProps;

var Ramp:array[byte]of byte;
    UseSat:boolean; SatFactor:double;

procedure PPCore_Resize;
var X,Y:integer;
begin
  if not Assigned(OriginalBMP) then begin
    if Assigned(ResizedBMP) then FreeAndNil(ResizedBMP);
    SizeX:=0; SizeY:=0;
    ImagePropsForm.SetLSize;
    exit;
  end;
  with ImagePropsForm do begin
    if SizeToWidth.Checked XOR Transforms.Transpose then begin
      X:=CSize.Position; Y:=round(X/SourceAspect);
    end else begin
      Y:=CSize.Position; X:=round(Y*SourceAspect);
    end;
    if Transforms.Transpose then begin
      SizeX:=Y; SizeY:=X;
    end else begin
      SizeX:=X; SizeY:=Y;
    end;
    SetLSize;
  end;
  if not Assigned(ResizedBMP) then ResizedBMP:=TBitmap.Create;
  if Transforms.Transpose then begin
    ResizedBMP.Width:=SizeY;
    ResizedBMP.Height:=SizeX;
  end else begin
    ResizedBMP.Width:=SizeX;
    ResizedBMP.Height:=SizeY;
  end;
  Scale(OriginalBMP,ResizedBMP);
end;

procedure PPCore_Transform;
var X,Y,DestX,DestY,IncX,IncY,StartX:integer;
    LCOT:boolean;  // "Local Copy Of Transpose", in der Hoffnung,
                   // da es eine Registervariable wird
    SrcLine,DestLine:PRGBLine;
    V:TRGB; Gray,I:integer;
begin
  if not Assigned(ResizedBMP) then begin
    if Assigned(BaseBMP) then FreeAndNil(BaseBMP);
    exit;
  end;
  if not Assigned(BaseBMP) then BaseBMP:=TBitmap.Create;
  BaseBMP.Width:=SizeX;
  BaseBMP.Height:=SizeY;
  BaseBMP.PixelFormat:=pf24bit;
  if Transforms.Mirror
    then begin StartX:=SizeX-1; IncX:=-1; end
    else begin StartX:=0; IncX:=1; end;
  if Transforms.Flip
    then begin DestY:=SizeY-1; IncY:=-1; end
    else begin DestY:=0; IncY:=1; end;
  LCOT:=Transforms.Transpose;
  SrcLine:=nil;
  for Y:=0 to (SizeY-1) do begin
    DestLine:=BaseBMP.ScanLine[DestY];
    if not LCOT then SrcLine:=ResizedBMP.ScanLine[Y];
    DestX:=StartX;
    for X:=0 to (SizeX-1) do begin
      if LCOT then begin
        SrcLine:=ResizedBMP.ScanLine[X];
        V:=SrcLine[Y];
      end else
        V:=SrcLine[X];
      if UseSat then begin
        Gray:=(integer(V.R)+integer(V.G)+integer(V.B)) DIV 3;
        I:=Gray+round((integer(V.R)-Gray)*SatFactor);
        if I>255 then V.R:=255 else if I<0 then V.R:=0 else V.R:=I;
        I:=Gray+round((integer(V.G)-Gray)*SatFactor);
        if I>255 then V.G:=255 else if I<0 then V.G:=0 else V.G:=I;
        I:=Gray+round((integer(V.B)-Gray)*SatFactor);
        if I>255 then V.B:=255 else if I<0 then V.B:=0 else V.B:=I;
      end;
      V.R:=Ramp[V.R]; V.G:=Ramp[V.G]; V.B:=Ramp[V.B];
      DestLine[DestX]:=V;
      inc(DestX,IncX);
    end;
    inc(DestY,IncY);
  end;
end;

procedure SetColorRamp(Factor,Brightness,Contrast,Gamma,Saturation:integer);
var i,v:integer; d,ContrastFactor,InvGamma:double;
begin
  if Contrast<=0 then ContrastFactor:=1.0+Contrast*0.01
  else if Contrast=100 then ContrastFactor:=256.0
  else ContrastFactor:=100.0/(100-Contrast);
  UseSat:=(Saturation<>0);
  if UseSat then begin
    if Saturation<=0 then SatFactor:=1.0+Saturation*0.01
    else if Saturation=100 then SatFactor:=256.0
    else SatFactor:=100.0/(100-Saturation);
  end;
  InvGamma:=100.0/Gamma;
  for i:=0 to 255 do begin
    if Factor<0 then d:=i*(100+Factor) DIV 100
    else if Factor>0 then d:=(i*(100-Factor)+255*Factor) DIV 100
    else d:=i;
    d:=(d+Brightness)/255.0;
    d:=0.5+(d-0.5)*ContrastFactor;
    if d<=0.0 then d:=0.0 else d:=exp(InvGamma*ln(d));
    try v:=round(255.0*d); except v:=0; end;
    if v<0 then v:=0; if v>255 then v:=255;
    Ramp[i]:=v;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
procedure PPCore_Reduce;
var Colors:array of TRGB;
    Indexes:array of byte;
    X,Y,C,MaxX,MaxY,MaxC,LinOfs:integer;
    Line,NextLine:PRGBLine; V,NV:TRGB;
    ThisDist,LeastDist,LeastDistAt,Hamming:integer;
    dR,dG,dB:integer;
  procedure Diffuse(var Dest:TRGB;Factor:integer);
  var I:integer;
  begin
    I:=Dest.R+dR*Factor DIV 16;
    if I<0 then Dest.R:=0 else if I>255 then Dest.R:=255 else Dest.R:=I;
    I:=Dest.G+dG*Factor DIV 16;
    if I<0 then Dest.G:=0 else if I>255 then Dest.G:=255 else Dest.G:=I;
    I:=Dest.B+dB*Factor DIV 16;
    if I<0 then Dest.B:=0 else if I>255 then Dest.B:=255 else Dest.B:=I;
  end;
begin
  if not Assigned(BaseBMP) then begin
    if Assigned(ReducedBMP) then FreeAndNil(ReducedBMP);
    exit;
  end;
  if not Assigned(ReducedBMP) then ReducedBMP:=TBitmap.Create;
  ReducedBMP.Width:=SizeX;
  ReducedBMP.Height:=SizeY;
  ReducedBMP.PixelFormat:=pf24bit;
  ReducedBMP.Canvas.Draw(0,0,BaseBMP);
  SetLength(PPData,SizeX*SizeY);
  // Ausgewhlte Farben zhlen
  MaxC:=0;
  for C:=0 to 255 do if PPColors[C].EntryType=etSelected then inc(MaxC);
  SetLength(Colors,MaxC); SetLength(Indexes,MaxC);
  // Ausgewhlte Farben eintragen
  MaxC:=-1;
  for C:=0 to 255 do if PPColors[C].EntryType=etSelected then begin
    inc(MaxC);
    Colors[MaxC]:=PPColors[C].Color;
    Indexes[MaxC]:=C;
  end;
  // "Hamming-Abstand" aller Farben bestimmen -- kann spter Zeit sparen
  Hamming:=maxint;
  for X:=0 to MaxC do with Colors[X] do
    for Y:=0 to MaxC do begin
      ThisDist:=sqr(Colors[Y].R-R)+sqr(Colors[Y].G-G)+sqr(Colors[Y].B-B);
      if ThisDist<Hamming then Hamming:=ThisDist;
    end;
  // alles durchgehen
  MaxX:=SizeX-1; MaxY:=SizeY-1;
  LinOfs:=0;
  NextLine:=ReducedBMP.ScanLine[0];
  for Y:=0 to MaxY do begin
    Line:=NextLine;
    if Y<MaxY then NextLine:=ReducedBMP.ScanLine[Y+1];
    for X:=0 to MaxX do begin
      V:=Line[X];
      // Farbe suchen
      LeastDist:=maxint; LeastDistAt:=0;
      for C:=0 to MaxC do begin
        with Colors[C] do
          ThisDist:=sqr(V.R-R)+sqr(V.G-G)+sqr(V.B-B);
        if ThisDist<LeastDist then begin
          LeastDist:=ThisDist;
          LeastDistAt:=C;
        end;
        if ThisDist<=Hamming then break;
      end;
      // Werte eintragen
      NV:=Colors[LeastDistAt];
      Line[X]:=NV;
      PPData[LinOfs]:=Indexes[LeastDistAt]; inc(LinOfs);
      // Diffusion Dither anwenden
      dR:=V.R-NV.R; dG:=V.G-NV.G; dB:=V.B-NV.B;
      if X<MaxX then Diffuse(Line[X+1],7);
      if Y<MaxY then begin
        if X>0 then Diffuse(NextLine[X-1],3);
        Diffuse(NextLine[X],5);
        if X<MaxX then Diffuse(NextLine[X+1],1);
      end;
    end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
procedure PPCore_Finish;
var NewPitch,NewOffset,NewSize:integer;
    X,Y,C:integer; s5:string[5];
    Ps:^byte; Pd:^char;
    LTemp:TPPLegendEntry;
    LongestName:integer;
    s,fmt:string;
    PercentualFactor:double;
begin
  if not Assigned(ReducedBMP) then begin
    SetLength(PPGrid,0);
    PPLegendMax:=0;
    exit;
  end;
  NewPitch:=(SizeX+7) AND $FFFFFFFC;
  NewOffset:=NewPitch+3;
  NewSize:=NewPitch*SizeY+SizeX+4;
  // PPGrid vorbereiten (Header etc.)
  if (NewPitch<>PPGPitch) OR (NewOffset<>PPGOffset) OR (NewSize<>length(PPGrid)) then begin
    PPGPitch:=NewPitch; PPGOffset:=NewOffset; SetLength(PPGrid,NewSize);
    PPGrid[0]:='P'; PPGrid[1]:='P'; PPGrid[2]:=#32; PPGrid[SizeX+3]:=#0;
    for X:=0 to (SizeX-1) do begin
      Y:=X MOD 5;
      if Y=0 then begin
        str(X DIV 5,s5);
        while s5[0]<#5 do begin inc(s5[0]); s5[byte(s5[0])]:=#32; end;
      end;
      PPGrid[X+3]:=s5[Y+1];
    end;
    for Y:=0 to (SizeY-1) do begin
      X:=PPGPitch*(Y+1);
      if (Y MOD 5)=0 then begin
        str((Y DIV 5):2,s5);
        PPGrid[X]:=s5[1]; PPGrid[X+1]:=s5[2];
      end else begin
        PPGrid[X]:=#32; PPGrid[X+1]:=#32;
      end;
      PPGrid[X+2]:=#32;
      PPGrid[X+SizeX+3]:=#0;
    end;
  end;
  // Legende vorbereiten & PaperPic ausfllen
  for C:=0 to 255 do with PPLegend[C] do begin Index:=C; Count:=0; end;
  Ps:=@PPData[0];
  for Y:=0 to (SizeY-1) do begin
    Pd:=@PPGrid[PPGPitch*(Y+1)+3];
    for X:=0 to (SizeX-1) do begin
      C:=Ps^; inc(Ps);
      inc(PPLegend[C].Count);
      Pd^:=PPColors[C].Character; inc(Pd);
    end;
  end;
  // Legende zusammenfassen
  PPLegendMax:=0; LongestName:=0;
  for C:=0 to 255 do
    if PPLegend[C].Count>0 then begin
      PPLegend[PPLegendMax]:=PPLegend[C];
      inc(PPLegendMax);
      X:=length(PPColors[PPLegend[C].Index].Name);
      if X>LongestName then LongestName:=X;
    end;
  // Legende sortieren
  for X:=0 to (PPLegendMax-2) do
    for Y:=(X+1) to (PPLegendMax-1) do
      if PPLegend[X].Count<PPLegend[Y].Count then begin
        LTemp:=PPLegend[X];
        PPLegend[X]:=PPLegend[Y];
        PPLegend[Y]:=LTemp;
      end;
  // Legende ASCII-mig formatieren
  SetLength(PPLegendStr,PPLegendMax+1);
  s:=Format('Gre: %.1f cm x %.1f cm',[SizeX*0.1,SizeY*0.1]);
  if length(s)>LongestName then LongestName:=length(s);
  fmt:='%s  %-'+IntToStr(LongestName)+'s  %'+
       IntToStr(length(IntToStr(SizeX*SizeY)))+'d Pixel = %6.2f%%';
  PPLegendStr[0]:=Format(fmt,['*',s,SizeX*SizeY,100.0]);
  PercentualFactor:=100.0/SizeX/SizeY;
  for C:=1 to PPLegendMax do
    with PPLegend[C-1],PPColors[Index] do
      PPLegendStr[C]:=Format(fmt,[Character,Name,Count,Count*PercentualFactor]);
end;

////////////////////////////////////////////////////////////////////////////////
var i:integer;
initialization
  for i:=0 to 255 do Ramp[i]:=i;
end.
