unit PSExport;
interface
uses SysUtils, Dialogs, PSTools, PPControl;

var PSExBoxLineWidth:double=0.75;
    PSExInnerLineWidth:double=0.25;
    PSExGray:double=0.0;
    PSExScale:double=1.0;
    PSExLegendFontSize:integer=10;
    PSExCenter:boolean=false;
    PSExReverseOrder:boolean=false;

type TPageInfo=record
                 PosX,PosY:integer;
                 StartX,StartY:integer;
                 RangeX,RangeY:integer;
                 X1,Y1,X2,Y2:double;
                 PVX,PVY:integer;  // frs Preview
               end;
type TLayoutInfo=record
                   BlocksX,BlocksY:integer;
                   PagesX,PagesY:integer;
                   BPPX,BPPY:integer;  // BPP = "Blocks Per Page"
                   BOLPX,BOLPY:integer;  // BOLP = "Blocks On Last Page"
                   Pages:array of TPageInfo;
                 end;

var PSExLayout:TLayoutInfo;
procedure PSExGetLayout;

procedure DoPSExport(const FileName:string);

implementation

function AlphaEnum(i:integer):string;
begin
  if i<26 then Result:=char(i+65)
          else Result:=char((i DIV 26)+64)+char((i MOD 26)+65);
end;

procedure RenderPage(X,Y:integer);
begin with PSExLayout,Pages[Y*PagesX+X] do begin
  if X<26 then NextPage(char(X+65)+IntToStr(Y+1))
          else NextPage(char((X DIV 26)+64)+char((X MOD 26)+65)+IntToStr(Y+1));
  writeln(PS,StartX,#32,StartY,#32,RangeX,#32,RangeY,#32,
             X1:0:1,#32,(PageSizeY-Y1):0:1,' PPDraw showpage');
end; end;

procedure DoPSExport(const FileName:string);
var X,Y:integer; PercentualFactor:double;
begin
  if (length(PPGrid)<=0) OR (PPLegendMax<=0) then exit;
  if not CreatePS(FileName,[psPageSize,psANSI,psPPLogo,psTextTools,psPPRenderer]) then begin
    MessageDlg('Konnte Datei'^M#34+FileName+#34^M'nicht schreiben!',mtError,[mbOK],0);
    exit;
  end;
  writeln(PS,'/Courier /Courier-ANSI ANSIEncode');
  writeln(PS,'/Helvetica /Helvetica-ANSI ANSIEncode');
  writeln(PS,'/Helvetica-BoldOblique /Helvetica-BoldOblique-ANSI ANSIEncode');
  writeln(PS,'/SizeX ',SizeX,' def /SizeY ',SizeY,' def /PPData [');
  for Y:=1 to SizeY do
    writeln(PS,'(',PSEscape(PChar(@PPGrid[Y*PPGPitch+3])),')');
  writeln(PS,'] def /PPLegend [');
  writeln(PS,'[(*)(Gre: ',SizeX*0.1:0:1,' cm x ',SizeY*0.1:0:1,' cm)(',
             SizeX*SizeY,')(100.00%)]');
  PercentualFactor:=100.0/SizeX/SizeY;
  for Y:=1 to PPLegendMax do
    with PPLegend[Y-1],PPColors[Index] do
      writeln(PS,'[(',PSEscape(Character),')(',PSEscape(Name),')(',Count,')(',
                 Count*PercentualFactor:0:2,'%)]');
  writeln(PS,'] def');
  PSExGetLayout;
  writeln(PS,'/PPBoxLineWidth ',PSExBoxLineWidth:0:2,' def');
  writeln(PS,'/PPInnerLineWidth ',PSExInnerLineWidth:0:2,' def');
  writeln(PS,'/PPGray ',PSExGray:0:2,' def');
  writeln(PS,'/PPScale ',PSExScale:0:2,' def');
  writeln(PS,'/PPLegendFontSize ',PSExLegendFontSize,' def');
  with PSExLayout do if PSExReverseOrder
    then for X:=0 to (PagesX-1) do for Y:=0 to (PagesY-1) do RenderPage(X,Y)
    else for Y:=0 to (PagesY-1) do for X:=0 to (PagesX-1) do RenderPage(X,Y);
  NextPage('Legende');
  writeln(PS,MarginX,#32,PageSizeY-MarginY,' PPDrawLegend showpage');
  ClosePS;
end;

procedure ClearLayout;
begin with PSExLayout do begin
  BlocksX:=0; BlocksY:=0;
  PagesX:=0; PagesY:=0;
  BPPX:=0; BPPY:=0;
  BOLPX:=0; BOLPY:=0;
  SetLength(Pages,0);
end; end;

procedure PSExGetLayout;
var X,Y:integer; SpanX,SpanY:double;
begin with PSExLayout do begin
  if (length(PPGrid)<=0) OR (PPLegendMax<=0) then begin
    ClearLayout; exit;
  end;
  BlocksX:=(SizeX+4) DIV 5;
  BlocksY:=(SizeY+4) DIV 5;
  BPPX:=trunc((PageSizeX-2*MarginX-13.0*PSExScale)/(26.0*PSExScale));
  BPPY:=trunc((PageSizeY-2*MarginY-10.0*PSExScale)/(40.0*PSExScale));
  PagesX:=(BlocksX+BPPX-1) DIV BPPX;
  PagesY:=(BlocksY+BPPY-1) DIV BPPY;
  BOLPX:=BlocksX-(PagesX-1)*BPPX;
  BOLPY:=BlocksY-(PagesY-1)*BPPY;
  SetLength(Pages,PagesX*PagesY);
  for Y:=0 to (PagesY-1) do
    for X:=0 to (PagesX-1) do
      with Pages[Y*PagesX+X] do begin
        PosX:=X; PosY:=Y;
        StartX:=X*BPPX; StartY:=Y*BPPY;
        RangeX:=BlocksX-StartX; if RangeX>BPPX then RangeX:=BPPX;
        RangeY:=BlocksY-StartY; if RangeY>BPPY then RangeY:=BPPY;
        SpanX:=(13.0+26.0*RangeX)*PSExScale;
        SpanY:=(10.0+40.0*RangeY)*PSExScale;
        if PSExCenter then begin
          X1:=0.5*(PageSizeX-SpanX);
          Y1:=0.5*(PageSizeY-SpanY);
        end else begin
          X1:=MarginX; Y1:=MarginY;
        end;
        X2:=X1+SpanX; Y2:=Y1+SpanY;
      end;
end; end;

initialization
  ClearLayout;
finalization
  Finalize(PSExLayout.Pages);
end.
