unit Preview;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ToolWin, ComCtrls, ExtCtrls, ImgList, PPControl;

type
  TPreviewForm = class(TForm)
    MainMenu: TMainMenu;
    MFile: TMenuItem;
    MQuit: TMenuItem;
    FunctionToolBar: TToolBar;
    Scroller: TScrollBox;
    ViewOriginal: TToolButton;
    ImageBox: TPaintBox;
    ViewBase: TToolButton;
    ViewReduced: TToolButton;
    Imagery: TImageList;
    ZoomOut: TToolButton;
    ZoomIn: TToolButton;
    ToolButton1: TToolButton;
    ShowImageProps: TToolButton;
    ShowColors: TToolButton;
    ViewGrid: TToolButton;
    ViewLegend: TToolButton;
    ToolButton4: TToolButton;
    MEdit: TMenuItem;
    MView: TMenuItem;
    MShowImageProps: TMenuItem;
    MShowColors: TMenuItem;
    MViewOriginal: TMenuItem;
    MViewBase: TMenuItem;
    MViewReduced: TMenuItem;
    MViewGrid: TMenuItem;
    MViewLegend: TMenuItem;
    MOpenImage: TMenuItem;
    MCloseImage: TMenuItem;
    MExportClassic: TMenuItem;
    N1: TMenuItem;
    MExportMM: TMenuItem;
    MExportPS: TMenuItem;
    DOpenImage: TOpenDialog;
    DExportClassic: TSaveDialog;
    MPageSetup: TMenuItem;
    DExportMM: TSaveDialog;
    DFancyExport: TSaveDialog;
    MExport: TMenuItem;
    N2: TMenuItem;
    MFancyExport: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MQuitClick(Sender: TObject);
    procedure ImageBoxPaint(Sender: TObject);
    procedure ViewOriginalClick(Sender: TObject);
    procedure ViewBaseClick(Sender: TObject);
    procedure ViewReducedClick(Sender: TObject);
    procedure ZoomOutClick(Sender: TObject);
    procedure ZoomInClick(Sender: TObject);
    procedure ShowImagePropsClick(Sender: TObject);
    procedure ShowColorsClick(Sender: TObject);
    procedure ViewGridClick(Sender: TObject);
    procedure ViewLegendClick(Sender: TObject);
    procedure MEditClick(Sender: TObject);
    procedure MViewClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MCloseImageClick(Sender: TObject);
    procedure MOpenImageClick(Sender: TObject);
    procedure MFileClick(Sender: TObject);
    procedure MExportClassicClick(Sender: TObject);
    procedure MExportMMClick(Sender: TObject);
    procedure MPageSetupClick(Sender: TObject);
    procedure MExportPSClick(Sender: TObject);
    procedure MExportClick(Sender: TObject);
    procedure MFancyExportClick(Sender: TObject);
  private
    { Private declarations }
    LH:integer;
  public
    { Public declarations }
    ViewMode:TViewMode;
    Zoom:integer;
    procedure SetViewMode(NewViewMode:TViewMode;NewZoom:integer);
  end;

var
  PreviewForm: TPreviewForm;
  EnableFancyExport:boolean=false;

implementation

uses ImageProps, Colors, OPTNClassic, PageSetup, OPTNmm, OPTNps, FancyExport;

{$R *.dfm}

procedure TPreviewForm.FormCreate(Sender: TObject);
begin
//AllocConsole;
  DecimalSeparator:='.';
  ViewMode:=vmOriginal; Zoom:=2;
  LoadColors;
  if EnableFancyExport then MFancyExport.Visible:=true;
  ImageBox.ControlStyle:=ImageBox.ControlStyle+[csOpaque];
end;

procedure TPreviewForm.FormDestroy(Sender: TObject);
begin
  SaveColors;
end;

procedure TPreviewForm.MQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TPreviewForm.SetViewMode(NewViewMode:TViewMode;NewZoom:integer);
var SX,SY,MyZoom:integer; NoZoomPlease:boolean;
begin
  ViewMode:=NewViewMode;
  Zoom:=NewZoom; if Zoom<1 then Zoom:=1;
  LH:=ImageBox.Canvas.TextHeight('X');
  SX:=0; SY:=0;
  case ViewMode of
    vmOriginal:if Assigned(OriginalBMP) then with OriginalBMP do
               begin SX:=Width; SY:=Height; end;
    vmBase:if Assigned(BaseBMP) then with BaseBMP do
           begin SX:=Width; SY:=Height; end;
    vmReduced:if Assigned(ReducedBMP) then with ReducedBMP do
              begin SX:=Width; SY:=Height; end;
    vmGrid:if length(PPGrid)>0 then with ImageBox.Canvas do
           begin SX:=TextWidth(PChar(@PPGrid[0])); SY:=LH*(SizeY+1); end;
    vmLegend:if length(PPGrid)>0 then with ImageBox.Canvas do
             begin SX:=TextWidth(PPLegendStr[0]); SY:=LH*(PPLegendMax+1); end;
  end;
  if ViewMode IN [vmGrid,vmLegend]
    then Scroller.Color:=clWindow
    else Scroller.Color:=clAppWorkSpace;
  NoZoomPlease:=(ViewMode IN [vmOriginal,vmGrid,vmLegend]) OR (SX<1) OR (SY<1);
  ZoomIn.Enabled:=not(NoZoomPlease);
  ZoomOut.Enabled:=not(NoZoomPlease);
  if NoZoomPlease then MyZoom:=1 else MyZoom:=Zoom;
  if (SX<1) OR (SY<1) then
    ImageBox.Visible:=false
  else begin
    ImageBox.Visible:=true;
    ImageBox.Width:=SX*MyZoom;
    ImageBox.Height:=SY*MyZoom;
  end;
  ImageBox.Invalidate;
end;

procedure TPreviewForm.ImageBoxPaint(Sender: TObject);
var Y:integer;
begin
  with ImageBox,Canvas do begin
    if Zoom=1 then
      case ViewMode of
        vmOriginal:if Assigned(OriginalBMP) then Draw(0,0,OriginalBMP);
        vmBase:if Assigned(BaseBMP) then Draw(0,0,BaseBMP);
        vmReduced:if Assigned(ReducedBMP) then Draw(0,0,ReducedBMP);
        vmGrid:for Y:=0 to SizeY do TextOut(0,LH*Y,PChar(@PPGrid[Y*PPGPitch]));
        vmLegend:for Y:=0 to PPLegendMax do TextOut(0,LH*Y,PPLegendStr[Y]);
      end
    else
      case ViewMode of
        vmOriginal:if Assigned(OriginalBMP) then
                   StretchDraw(Rect(0,0,Width,Height),OriginalBMP);
        vmBase:if Assigned(BaseBMP) then
                   StretchDraw(Rect(0,0,Width,Height),BaseBMP);
        vmReduced:if Assigned(ReducedBMP) then
                   StretchDraw(Rect(0,0,Width,Height),ReducedBMP);
        vmGrid:for Y:=0 to SizeY do TextOut(0,LH*Y,PChar(@PPGrid[Y*PPGPitch]));
        vmLegend:for Y:=0 to PPLegendMax do TextOut(0,LH*Y,PPLegendStr[Y]);
      end;
  end;
end;

procedure TPreviewForm.ViewOriginalClick(Sender: TObject);
begin
  SetViewMode(vmOriginal,Zoom);
  ViewOriginal.Down:=true;
end;

procedure TPreviewForm.ViewBaseClick(Sender: TObject);
begin
  SetViewMode(vmBase,Zoom);
  ViewBase.Down:=true;
end;

procedure TPreviewForm.ViewReducedClick(Sender: TObject);
begin
  SetViewMode(vmReduced,Zoom);
  ViewReduced.Down:=true;
end;

procedure TPreviewForm.ViewGridClick(Sender: TObject);
begin
  SetViewMode(vmGrid,Zoom);
  ViewGrid.Down:=true;
end;

procedure TPreviewForm.ViewLegendClick(Sender: TObject);
begin
  SetViewMode(vmLegend,Zoom);
  ViewLegend.Down:=true;
end;

procedure TPreviewForm.ZoomOutClick(Sender: TObject);
begin
  if Zoom>1 then SetViewMode(ViewMode,Zoom-1) else Beep;
end;

procedure TPreviewForm.ZoomInClick(Sender: TObject);
begin
  SetViewMode(ViewMode,Zoom+1);
end;

procedure TPreviewForm.ShowImagePropsClick(Sender: TObject);
begin
  with ImagePropsForm do Visible:=not(Visible);
end;

procedure TPreviewForm.ShowColorsClick(Sender: TObject);
begin
  with ColorsForm do Visible:=not(Visible);
end;

procedure TPreviewForm.MEditClick(Sender: TObject);
begin
  MShowImageProps.Checked:=ImagePropsForm.Visible;
  MShowColors.Checked:=ColorsForm.Visible;
end;

procedure TPreviewForm.MViewClick(Sender: TObject);
begin
  case ViewMode of
    vmOriginal:MViewOriginal.Checked:=true;
    vmBase:MViewBase.Checked:=true;
    vmReduced:MViewReduced.Checked:=true;
    vmGrid:MViewGrid.Checked:=true;
    vmLegend:MViewLegend.Checked:=true;
  end;
end;

procedure TPreviewForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Application.Terminate;
end;

procedure TPreviewForm.MCloseImageClick(Sender: TObject);
begin
  LoadImage('');
end;

procedure TPreviewForm.MOpenImageClick(Sender: TObject);
begin
  if not DOpenImage.Execute then exit;
  if not LoadImage(DOpenImage.FileName) then
    MessageDlg('Konnte Bilddatei'^M#34+DOpenImage.FileName+#34^M'nicht ffnen!',
               mtError,[mbOK],0);
end;

procedure TPreviewForm.MFileClick(Sender: TObject);
begin
  MCloseImage.Enabled:=Assigned(OriginalBMP);
end;

procedure TPreviewForm.MExportClassicClick(Sender: TObject);
begin
  if (length(PPGrid)<=0) OR (PPLegendMax<=0) then exit;
  if not DExportClassic.Execute then exit;
  OPTNClassicForm.FileName:=DExportClassic.FileName;
  OPTNClassicForm.ShowModal;
end;

procedure TPreviewForm.MExportMMClick(Sender: TObject);
begin
  if (SizeX<1) OR (SizeY<1) then exit;
  if not DExportMM.Execute then exit;
  OPTNmmForm.FileName:=DExportMM.FileName;
  OPTNmmForm.ShowModal;
end;

procedure TPreviewForm.MPageSetupClick(Sender: TObject);
begin
  with PageSetupForm do Visible:=not(Visible);
end;

procedure TPreviewForm.MExportPSClick(Sender: TObject);
begin
  with OPTNpsForm do Visible:=not(Visible);
end;

procedure TPreviewForm.MExportClick(Sender: TObject);
begin
  MPageSetup.Checked:=PageSetupForm.Visible;
  MExportPS.Checked:=OPTNpsForm.Visible;
  MExportMM.Enabled:=(SizeX>0) AND (SizeY>0);
  MExportClassic.Enabled:=(length(PPGrid)>0) AND (PPLegendMax>0);
  MFancyExport.Enabled:=Assigned(ReducedBMP);
end;

procedure TPreviewForm.MFancyExportClick(Sender: TObject);
begin
  if not Assigned(ReducedBMP) then exit;
  if not DFancyExport.Execute then exit;
  if UpperCase(ExtractFileExt(DFancyExport.FileName))='.BMP'
    then FancyExport.XP_BMP(DFancyExport.FileName)
    else FancyExport.XP_JASC_PAL(DFancyExport.FileName);
end;

end.
